
class NestedGuiElement:
    '''
        It is not an element container. It means it can not be find elements within itself.
    '''

    def __init__(self, automator, parent_element, emd):
        pass

    
